#include "SocketConversation.h"
#include "SocketServeurSimple.h"
#include <iostream>
#include <string>
using std::cout;
using std::endl;

unsigned long __stdcall Envoyer     (void*);
unsigned long __stdcall Recevoir    (void*);

SocketConversation::SocketConversation
   (const socket_t sck, volatile SocketServeurSimple *parent)
   : socket_(sck), parent_(parent),
     hEnvoyer_(0), hRecevoir_(0)
{
}

void SocketConversation::demarrer()
{
   if (!(hEnvoyer_ || hRecevoir_))
   {
      hEnvoyer_ = CreateThread (0, 0, ::Envoyer, this, 0, 0);
      hRecevoir_ = CreateThread (0, 0, ::Recevoir, this, 0, 0);
   }
}

SocketConversation::~SocketConversation() throw()
{
   if (hEnvoyer_ || hRecevoir_)
   {
      demander_arret();
      while (!est_arrete()) Sleep(1); // beurk;
      if (hEnvoyer_)
         CloseHandle(hEnvoyer_);
      if (hRecevoir_)
         CloseHandle(hRecevoir_);
   }
   fermer();
}

void SocketConversation::fermer() throw()
{
   if (!est_invalide(socket_))
   {
      fermer_socket(socket_);
      invalider_socket(socket_);
   }
}

void SocketConversation::envoyer() volatile
{
   using std::string;
   // pour se souvenir de ce qui n'a pas t envoy prcdemment
   string &s = const_cast<string &>(en_cours_);
   s += a_envoyer_.extraire();
   if (!s.empty())
   {
      int n = send
         (socket_, s.c_str(), static_cast<int>(s.size()), 0);
      if (est_erreur(n))
         demander_arret();
      else
         s = s.substr(n);
   }
}

std::string SocketConversation::recevoir() volatile
{
   const int TAILLE_TAMPON = 64; // taille arbitraire
   char tampon[TAILLE_TAMPON];
   int n = recv(socket_, tampon, TAILLE_TAMPON, 0);
   std::string s;
   if (!n || est_erreur(n))
      demander_arret();
   else
      parent_->ajouter_diffusion(s = std::string(tampon, tampon + n));
   return s;
}

unsigned long __stdcall Recevoir(void *p)
{
   SocketConversation *sc =
      reinterpret_cast<SocketConversation*>(p);
   do
   {
      std::string s = sc->recevoir();
      cout << "Reu: " << s << endl;
      Sleep(10);
   }
   while (!sc->arret_demande());
   sc->arret_complete();
   return 0UL;
}

unsigned long __stdcall Envoyer(void *p)
{
   SocketConversation *sc =
      reinterpret_cast<SocketConversation*>(p);
   do
   {
      sc->envoyer();
      Sleep(10);
   }
   while (!sc->arret_demande());
   sc->arret_complete();
   return 0UL;
}

