#include "SocketServeurSimple.h"
#include "SocketConversation.h"
#include "StrateIsolante.h"
#include <vector>
using std::vector;

SocketServeurSimple::SocketServeurSimple(const short port_denormalise)
   : socket_(creer_socket())
{
   if (est_invalide(socket_))
      exit(erreur("Cration"));

   // On rend le serveur disponible en rservant son port
   reserver_port(socket_, htons(port_denormalise));
}

#include <algorithm>
using std::for_each;
void SocketServeurSimple::fermer_conversations() volatile
{
   Autoverrou a(mutex_);
   struct demander_arret_conversations
   {
      void operator()(volatile SocketConversation *p)
         { p->demander_arret(); }
   };
   vector<SocketConversation *> &v =
      const_cast<vector<SocketConversation*>&>(conversations_);
   for_each(v.begin(), v.end(), demander_arret_conversations());
}

void SocketServeurSimple::vider_conversations() volatile
{
   Autoverrou a(mutex_);
   vector<SocketConversation*> &v =
      const_cast<vector<SocketConversation*>&>(conversations_);
   for (vector<SocketConversation *>::iterator itt = v.begin();
        itt != v.end(); )
   {
      SocketConversation *temp = *itt++;
      temp->demander_arret();
      while (!temp->est_arrete()) Sleep(1); // beurk
      delete temp;
   }
   v.clear();
}

void SocketServeurSimple::fermer() throw()
{
   if (!est_invalide(socket_))
   {
      fermer_socket(socket_);
      invalider_socket(socket_);
   }
}

SocketServeurSimple::~SocketServeurSimple() throw()
{
   vider_conversations();
   fermer();
}
void SocketServeurSimple::accepter() volatile
{
   socket_t pourJaser = accepter_client(socket_);
   if (est_invalide(pourJaser))
      demander_arret();
   else
   {
      SocketConversation *nouveau =
         new SocketConversation(pourJaser, this);
      if (nouveau)
      {
         Autoverrou a(mutex_);
         const_cast<vector<SocketConversation*>&>(conversations_).push_back(nouveau);
         nouveau->demarrer();
      }
   }
}

#include <algorithm>
using std::partition;
using std::for_each;
struct Meurs
{
   template <class T>
      void operator () (T p) throw()
      {
         delete p;
      }
};
void SocketServeurSimple::diffuser_nettoyer() volatile
{
   // Rpartir le texte  diffuser et liminer les sockets de
   // conversation dchus
   std::string a_distribuer = a_diffuser_.extraire();
   Autoverrou a(mutex_);
   vector<SocketConversation*> &v =
      const_cast<vector<SocketConversation*>&>(conversations_);
   if (!a_distribuer.empty())
   {
      // Diffuser
      for (vector<SocketConversation*>::iterator itt = v.begin(); itt != v.end(); ++itt)
         if (!(*itt)->est_arrete())
            (*itt)->ajouter_texte(a_distribuer);
   }
   // Nettoyer
   struct est_arrete
   {
      bool operator()(SocketConversation *p)
         { return p->est_arrete(); }
   };
   vector<SocketConversation*>::iterator seuil =
      partition(v.begin(), v.end(), est_arrete());
   for_each (v.begin (), seuil, Meurs());
   vector<SocketConversation*> vivants(seuil, v.end());
   v = vector<SocketConversation*>(seuil, v.end());
}
